"""navig_telegram — importable namespace for the navig-telegram pack.

This __init__.py makes the src/ directory importable as the ``navig_telegram``
package once ``navig-telegram``'s on_load has added src/ to sys.path.
"""
